import {Injectable} from '@angular/core';
import {Http, Headers, Response} from '@angular/http';
import {Observable} from 'rxjs/Observable';
import {Observer} from 'rxjs/Observer';
import 'rxjs/Rx';

@Injectable()
export class ConstructApi {
  private apiHost: string;
  private appId: string;
  private agency: string;
  private environment: string;
  private authToken: string;
  
  private _baseParams = {
    lang: 'en'
  }

  private _getParams = {
    limit: 100,
    offset: 0,
    lang: 'en'
  }

  constructor(private http: Http) { }

  private headers(customHeaders) {
    var httpHeaders = new Headers({
      'x-accela-appid': this.appId,
      'x-accela-agency': this.agency,
      'x-accela-environment': this.environment
    });
    
    if(this.authToken) {
      httpHeaders.append('Authorization', this.authToken);
    }
    
    if (customHeaders) {
      for (var key in customHeaders) {
        httpHeaders.append(key, customHeaders[key]);
      }
    }

    return httpHeaders;
  }

  private url(path, userParams?, baseParams = this._baseParams) {
    let params = baseParams;

    if (userParams) {
      Object.assign(params, userParams)
    }

    let urlParams = Object.keys(params).map((k) => {
      return `${k}=${encodeURI(params[k])}`;
    }).join('&');

    return `${this.apiHost}${path}?${urlParams}`;
  }
  
  setup(config: any) {
    this.appId = config.appId;
    this.apiHost = config.apiHost;
  }
  
  setAuthToken(authToken: string) {
    this.authToken = authToken;
  }
  
  setAgency(agency: string) {
    this.agency = agency;  
  }
  
  setEnvironment(environment: string) {
    this.environment = environment;  
  }

  get(path: string, urlParameters?, customHeaders?, maxItems = 500) {
    let self = this;
    let pageOffset = (urlParameters && urlParameters.offset) ? urlParameters.offset : self._getParams.offset;

    return Observable.create((obs: Observer<any>) => {
      function getPage(offset, count = 0) {
        self.http.get(self.url(path, urlParameters, self._getParams), { headers: self.headers(customHeaders) })
          .map(res => res.json())
          .subscribe(
            (data) => {
              if (!data.result) {
                data.result = [];
              }
              
              obs.next(data.result);

              if (data.page && data.page.hasmore && count < maxItems) {
                getPage(data.page.limit + data.page.offset, count + data.result.length);
              } else {
                obs.complete();
              }
            },
            (error) => {
              obs.error(JSON.stringify(error));
            }
          );
      }

      getPage(pageOffset);
    })
    .flatMap((result) => {
      return result;
    })
  }

  batch(path, urlParams?, customHeaders?) {
    if (path instanceof Array) {
      var request = [];

      for (var i = 0; i < path.length; i++) {
        var tmpReq = {
          relativeUrl: path[i],
          method: 'GET'
        };

        request.push(tmpReq);
      }

      var requestData = JSON.stringify(request);
      return this.post('/v4/batch', requestData, null, null);
    }
  }

  download(path, urlParameters?, customHeaders?) {
    // this will error out
    // no blob response type implemented on angular 2 yet
    return this.http.get(this.url(path, urlParameters), { headers: this.headers(customHeaders) })
                    .map(res => res.blob() );
  }

  post(path:string, body, urlParameters?, customHeaders?) {
    return this.http.post(this.url(path, urlParameters), JSON.stringify(body), { headers: this.headers(customHeaders) })
                    .map(res => res.json()['result'] || []);
  }

  // like post but for searches where response can be in pages
  search(path: string, body, urlParameters?, customHeaders?, maxItems = 500) {
    let self = this;
    let pageOffset = (urlParameters && urlParameters.offset) ? urlParameters.offset : self._getParams.offset;

    if (urlParameters && urlParameters.limit) {
      maxItems = urlParameters.limit;
    }

    return Observable.create((obs: Observer<any>) => {
      function getPage(offset, count = 0) {
        self.http.post(self.url(path, urlParameters, self._getParams), JSON.stringify(body), { headers: self.headers(customHeaders) })
          .map(res => res.json())
          .subscribe(
          (data) => {
            if(!data.result) {
              data.result = [];
            }
            
            obs.next(data.result);
            count += data.result.length;

            if (data.page && data.page.hasmore && (count < maxItems)) {
              getPage(data.page.limit + data.page.offset, count);
            } else {
              obs.complete();
            }
          },
          (error) => {
            obs.error(error);
          }
          );
      }

      getPage(pageOffset);
    })
    .flatMap((result) => {
      return result;
    })
  }

  update(path:string, body, urlParameters?, customHeaders?) : Observable<any> {
    return this.http.put(this.url(path, urlParameters), JSON.stringify(body), { headers: this.headers(customHeaders) })
                    .map(res => res.json()['result'] || []);
  }

  del(path:string, urlParameters?, customHeaders?) {
    return this.http.delete(this.url(path, urlParameters), {headers: this.headers(customHeaders)})
                    .map(res => res.json()['result'] || []);
  }
}